#!/usr/bin/env python

## without plots

import sys
from dgstudent_noplot import *

########## user configurable parameters ########

## choice of solver and O(max number of iterations)
# localsolver:lbfgs,cg,neldermead,leastsq,root
# maxitn:50,10,1000,1000,1000
#localsolver = "root" 
#maxitn = 500
localsolver = "lbfgs"
maxitn = 10

# infinity
inf = 1e50

# zero
myZero = 1e-6

########## functions #########

## compute the Gram matrix from the distance matrix
def dist2Gram(EDM):
    n = EDM.shape[0]
    J = np.identity(n) - (1.0/n)*np.ones((n,n))
    B = -0.5 * np.dot(J,np.dot(np.square(EDM), J))
    return B

## classic multidimensional scaling
def MDS(B):
    n = B.shape[0]
    x = factor(B) # returns cols sorted by decreasing eigval
    (evals,evecs) = np.linalg.eigh(x)
    K = len(evals[evals > myZero])
    if K < n:
        # only first K columns (corresp to largest eigvals)
        x = x[:,0:K]
    return x
    
## perform PCA
def PCA(B,K):
    n = B.shape[0]
    x = factor(B) # returns cols sorted by decreasing eigval
    if K < n:
        # only first K columns (corresp to largest eigvals)
        x = x[:,0:K]
    return x

###################### MAIN #####################

## command line expects a _gph.dat file
if len(sys.argv) < 2:
    exit('need file_gph.dat on command line')

## read the instance from file
G = instanceFromFile(sys.argv[1]).graph
instancename = ''.join(os.path.basename(sys.argv[1]).split('_')[:-1])

n = G.V.card
K = 3
xorg = np.zeros((n,K))
ldeorg = -1
mdeorg = -1

## if given on command line, also read the realization from a _rlz.dat file
if len(sys.argv) >= 3:
    print "dgstudent: also reading a realization from", sys.argv[2]
    xorg = realizationFromRlzDatFile(G, sys.argv[2])
    ldeorg = LDE(G,xorg)
    mdeorg = MDE(G,xorg)

## complete the distance matrix using shortest path length
print "dgstudent: Floyd-Warshall algorithm..."
A = FloydWarshall(G)
## approximate Gram matrix
B = dist2Gram(A)

## multidimensional scaling
print "dgstudent: multidimensional scaling..."
xmds = MDS(B)
K2 = xmds.shape[1]
ldemds = LDE(G,xmds)
mdemds = MDE(G,xmds)
x1 = LocalNLPSolver(G, xmds, K2, {}, localsolver, maxitn)
x1 = PCA(x1.dot(x1.T),K)
lde1 = LDE(G,x1)
mde1 = MDE(G,x1)

## principal component analysis
xpca = PCA(B,K)   ## the 3 most significant dimensions
print "dgstudent: principal component analysis..."
ldepca = LDE(G,xpca)
mdepca = MDE(G,xpca)
x2 = LocalNLPSolver(G, xpca, K, {}, localsolver, maxitn)
lde2 = LDE(G,x2)
mde2 = MDE(G,x2)

## print results
print "                   MDS3          MDS3+NLP  PCA           PCA+NLP"
print "dgstudent:instance,ldemds,mdemds,lde1,mde1,ldepca,mdepca,lde2,mde2"
print "{0:s},{1:.2f},{2:.2f},{3:.2f},{4:.2f},{5:.2f},{6:.2f},{7:.2f},{8:.2f},{9:.2f},{10:.2f}".format(instancename,ldemds,mdemds,lde1,mde1,ldepca,mdepca,lde2,mde2,ldeorg,mdeorg)

